import { createSignal, onCleanup, onMount } from "solid-js";
import L from 'leaflet';
import { FaSolidLocationPin } from "solid-icons/fa";
import Navbar from "../partials/Navbar";

function Finder() {
  const [pos, setPos] = createSignal<[number, number]>([0, 0]);
  const [geo, setGeo] = createSignal(true);

  let markers: L.Marker[] = [];

  let first = true;
  let watchId: number = -1;
  let map: L.Map;

  let me = L.marker([0, 0]);

  /**
   * Montar Leafmap
   */
  onMount(() => {
    map = L.map('map');
    L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
      maxZoom: 19,
      attribution: '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
    }).addTo(map);

    if (geo()) {
      listenGeolocation();
    }
  });

  /**
   * Limpiar watch de geolocalización si
   * está activo
   */
  onCleanup(() => {
    if (watchId !== -1) {
      navigator.geolocation.clearWatch(watchId);
    }
  });

  /**
   * Activar / Desactivar uso de Geolocalización
   */
  const toggleGeo = () => {
    if (geo()) {
      navigator.geolocation.clearWatch(watchId);
      watchId = -1;
    } else {
      listenGeolocation();
    }

    setGeo(!geo());
  }

  /**
   * Activar listener de geolocalización en tiempo real
   */
  const listenGeolocation = () => {
    watchId = navigator.geolocation.watchPosition(
      (pos) => {
        setPos([pos.coords.latitude, pos.coords.longitude]);
        handleLocation();
      },
      (err) => {
        alert(err.message);
      }
    )
  }

  /**
   * Escribe los puntos relevantes
   * (Mi ubicación y los embalses cercanos)
   */
  const handleLocation = () => {
    me.setLatLng(pos());
    if (first) {
      map.setView(pos(), 9);
      me.addTo(map);
      first = false;
    }

    // Cleanup
    if (markers.length > 0) {
      markers.forEach((m) => {
        map.removeLayer(m);
      })

      markers = [];
    }

    // TODO: Hacer petición y sacar markers
  }

  /**
   * Escribe la latitud de la tupla manualmente
   * @param val Latitud dada por el input
   */
  const setLatitudeManual = (val: string) => {
    const lat = parseFloat(val);

    setPos([lat, pos()[1]]);
  }

  /**
   * Escribe la longitud de la tupla manualmente
   * @param val longitud dada por el input
   */
  const setLongitudeManual = (val: string) => {
    const lon = parseFloat(val);

    setPos([pos()[0], lon]);
  }

  return (
    <>
      <Navbar />
      <section class="section">
        <div class="container">
          <div class="columns is-centered is-vcentered is-multiline">
            <div class="column is-narrow">
              <div class="field has-addons has-addons-centered">
                <div class="control">
                  <input class="input" on:change={(e) => setLatitudeManual(e.target.value)} type="text" disabled={geo()} placeholder="Latitud" value={pos()[0]} />
                </div>
                <div class="control">
                  <input class="input" on:change={(e) => setLongitudeManual(e.target.value)} type="text" disabled={geo()} placeholder="Longitud" value={pos()[1]} />
                </div>
                <div class="control">
                  <button disabled={geo()} on:click={() => handleLocation()} class="button is-info">
                    Aplicar
                  </button>
                </div>
              </div>
            </div>
            <div class="column is-narrow">
              <button on:click={() => toggleGeo()} class="button" classList={{ 'is-success': geo(), 'is-danger': !geo() }}>
                <span class="icon">
                  <FaSolidLocationPin />
                </span>
                <span>Geolocalización</span>
              </button>
            </div>
          </div>
          <div id="map" style={{
            height: '70vh'
          }}></div>
        </div>
      </section>
    </>
  )
}

export default Finder;
